# Copyright (C) 2017-2018 StARLinG Lab
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program (at the base of this repository). If not,
# see <http://www.gnu.org/licenses/>

from .parse import *
from .corpus import readCorpus

import argparse
import logging

# === Metadata === #

__author__ = 'Kaushik Roy (@kkroy36)'
__copyright__ = 'Copyright (c) 2017-2018 StARLinG Lab'
__license__ = 'GPL-v3'

__version__ = '0.1.0'
__status__ = 'Beta'
__maintainer__ = 'Alexander L. Hayes (@batflyer)'
__email__ = 'alexander.hayes@utdallas.edu'

__credits__ = [
    'Kaushik Roy (@kkroy36)',
    'Alexander L. Hayes (@batflyer)',
    'Sriraam Natarajan (@boost-starai)',
    'Gautam Kunapuli (@gkunapuli)',
    'Dileep Viswanathan',
    'Rahul Pasunuri'
]

# === Logging === #

logger = logging.getLogger(__name__)
logger.setLevel(logging.INFO)

log_handler = logging.FileHandler('rnlp_log.log')
log_handler.setLevel(logging.INFO)
formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
log_handler.setFormatter(formatter)

logger.addHandler(log_handler)
logger.info('Started logger.')

# === Argument Parser === #

logger.info('Started Argument Parser.')
parser = argparse.ArgumentParser(
    description='''Relational-NLP: A library and tool for converting text
                   into a set of relational facts.''',
    epilog='''Copyright (c) 2017-2018 StARLinG Lab.'''
)

file_or_dir = parser.add_mutually_exclusive_group()

parser.add_argument('-b', '--blockSize', type=int, default=2,
    help='Set the block size')
file_or_dir.add_argument('-d', '--directory', type=str,
    help='Read text from all files in a directory.')
file_or_dir.add_argument('-f', '--file', type=str,
    help='Read from text from a file.')

args = parser.parse_args()
logger.info('Argument Parsing Successful.')

# Set block size.
n = args.blockSize
logger.info('blockSize specified as ' + str(n))

# Set the input file(s).
if args.file:
    chosenFile = args.file
elif args.directory:
    chosenFile = args.directory
else:
    message = 'Error. No file or directory was specified.'
    logger.error(message)
    print(message)
    exit(1)

# Read the corpus.
try:
    corpus = readCorpus(chosenFile)
except Exception:
    logger.error('Error while reading corpus.', exc_info=True)
    exit(2)

# Get sentences from the corpus.
try:
    sentences = getSentences(corpus)
except Exception:
    logger.error('Error getting sentences from corpus', exc_info=True)
    exit(2)

# Create blocks from the sentences.
try:
    blocks = getBlocks(sentences, n)
except Exception:
    logger.error('Error while creating blocks', exc_info=True)
    exit(2)

# Make identifiers from the blocks.
try:
    makeIdentifiers(blocks)
except Exception:
    logger.error('Error while making identifiers.', exc_info=True)
    exit(2)

logger.info('Reached bottom of ' + __name__ + '.')
logger.info('Shutting down logger.')
logging.shutdown()
exit(0)



# Copyright (C) 2017-2018 StARLinG Lab
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program (at the base of this repository). If not,
# see <http://www.gnu.org/licenses/>

"""
rnlp.corpus
-----------

Built-in corpus and utilities for reading corpora from files.

.. code-block:: python

                # rnlp.corpus is not imported by default.
                import rnlp.corpus
"""

from os import listdir
from tqdm  import tqdm

def readCorpus(location):
    """
    Returns the contents of a file or a group of files as a string.

    :param location: .txt file or a directory to read files from.
    :type location: str.

    :returns: A string of all contents joined together.
    :rtype: str.

    .. note::

        This function takes a ``location`` on disk as a parameter. Location is
        assumed to be a string representing a text file or a directory. A text
        file is further assumed to contain ``.txt`` as a file extension while
        a directory may be a path.

    Example:

    .. code-block:: python

                    from rnlp.corpus import readCorpus

                    # If you have a text file:
                    doi = readCorpus('files/doi.txt')

                    # If you have multiple files to read from:
                    corpus = readCorpus('files')
    """
    print("Reading corpus from file(s)...")

    corpus = ''

    if '.txt' in location:
        with open(location) as fp:
            corpus = fp.read()
    else:

        dirFiles = listdir(location)
        nFiles = len(dirFiles)

        for f in tqdm(dirFiles):
            with open(location+"/"+f) as fp:
                corpus += fp.read()

    return corpus

def declaration():
    """
    The 'Declaration of Independence' was created before US Copyright Law,
    and is part of the public domain. This version captures the main text, but
    leaves out the signatures.

    The text here is based on the text available from the US National Archives
    https://www.archives.gov/founding-docs/declaration-transcript

    .. code-block:: python

                    import rnlp
                    from rnlp import corpus

                    # Import the document from the corpus:
                    doi = corpus.declaration()

                    print(len(doi))
                    print(type(doi))
    """

    return """
    In Congress, July 4, 1776. The unanimous Declaration of the thirteen united
    States of America, When in the Course of human events, it becomes necessary
    for one people to dissolve the political bands which have connected them
    with another, and to assume among the powers of the earth, the separate and
    equal station to which the Laws of Nature and of Nature's God entitle them,
    a decent respect to the opinions of mankind requires that they should
    declare the causes which impel them to the separation.

    We hold these truths to be self-evident, that all men are created equal,
    that they are endowed by their Creator with certain unalienable Rights,
    that among these are Life, Liberty and the pursuit of Happiness.--That to
    secure these rights, Governments are instituted among Men, deriving their
    just powers from the consent of the governed, --That whenever any Form of
    Government becomes destructive of these ends, it is the Right of the People
    to alter or to abolish it, and to institute new Government, laying its
    foundation on such principles and organizing its powers in such form, as
    to them shall seem most likely to effect their Safety and Happiness.
    Prudence, indeed, will dictate that Governments long established should not
    be changed for light and transient causes; and accordingly all experience
    hath shewn, that mankind are more disposed to suffer, while evils are
    sufferable, than to right themselves by abolishing the forms to which they
    are accustomed. But when a long train of abuses and usurpations, pursuing
    invariably the same Object evinces a design to reduce them under absolute
    Despotism, it is their right, it is their duty, to throw off such
    Government, and to provide new Guards for their future security.--Such has
    been the patient sufferance of these Colonies; and such is now the
    necessity which constrains them to alter their former Systems of
    Government. The history of the present King of Great Britain is a history
    of repeated injuries and usurpations, all having in direct object the
    establishment of an absolute Tyranny over these States. To prove this, let
    Facts be submitted to a candid world.

    He has refused his Assent to Laws, the most wholesome and necessary for the
    public good.

    He has forbidden his Governors to pass Laws of immediate and pressing
    importance, unless suspended in their operation till his Assent should be
    obtained; and when so suspended, he has utterly neglected to attend to them.

    He has refused to pass other Laws for the accommodation of large districts
    of people, unless those people would relinquish the right of Representation
    in the Legislature, a right inestimable to them and formidable to tyrants
    only.

    He has called together legislative bodies at places unusual, uncomfortable,
    and distant from the depository of their public Records, for the sole
    purpose of fatiguing them into compliance with his measures.

    He has dissolved Representative Houses repeatedly, for opposing with manly
    firmness his invasions on the rights of the people.

    He has refused for a long time, after such dissolutions, to cause others to
    be elected; whereby the Legislative powers, incapable of Annihilation, have
    returned to the People at large for their exercise; the State remaining in
    the mean time exposed to all the dangers of invasion from without, and
    convulsions within.

    He has endeavoured to prevent the population of these States; for that
    purpose obstructing the Laws for Naturalization of Foreigners; refusing to
    pass others to encourage their migrations hither, and raising the
    conditions of new Appropriations of Lands.

    He has obstructed the Administration of Justice, by refusing his Assent to
    Laws for establishing Judiciary powers.

    He has made Judges dependent on his Will alone, for the tenure of their
    offices, and the amount and payment of their salaries.

    He has erected a multitude of New Offices, and sent hither swarms of
    Officers to harrass our people, and eat out their substance.

    He has kept among us, in times of peace, Standing Armies without the
    Consent of our legislatures.

    He has affected to render the Military independent of and superior to the
    Civil power.

    He has combined with others to subject us to a jurisdiction foreign to our
    constitution, and unacknowledged by our laws; giving his Assent to their
    Acts of pretended Legislation:

    For Quartering large bodies of armed troops among us:

    For protecting them, by a mock Trial, from punishment for any Murders which
    they should commit on the Inhabitants of these States:

    For cutting off our Trade with all parts of the world:

    For imposing Taxes on us without our Consent:

    For depriving us in many cases, of the benefits of Trial by Jury:

    For transporting us beyond Seas to be tried for pretended offences

    For abolishing the free System of English Laws in a neighbouring Province,
    establishing therein an Arbitrary government, and enlarging its Boundaries
    so as to render it at once an example and fit instrument for introducing
    the same absolute rule into these Colonies:

    For taking away our Charters, abolishing our most valuable Laws, and
    altering fundamentally the Forms of our Governments:

    For suspending our own Legislatures, and declaring themselves invested
    with power to legislate for us in all cases whatsoever.

    He has abdicated Government here, by declaring us out of his Protection and
    waging War against us.

    He has plundered our seas, ravaged our Coasts, burnt our towns, and
    destroyed the lives of our people.

    He is at this time transporting large Armies of foreign Mercenaries to
    compleat the works of death, desolation and tyranny, already begun with
    circumstances of Cruelty & perfidy scarcely paralleled in the most
    barbarous ages, and totally unworthy the Head of a civilized nation.

    He has constrained our fellow Citizens taken Captive on the high Seas to
    bear Arms against their Country, to become the executioners of their
    friends and Brethren, or to fall themselves by their Hands.

    He has excited domestic insurrections amongst us, and has endeavoured to
    bring on the inhabitants of our frontiers, the merciless Indian Savages,
    whose known rule of warfare, is an undistinguished destruction of all ages,
    sexes and conditions.

    In every stage of these Oppressions We have Petitioned for Redress in the
    most humble terms: Our repeated Petitions have been answered only by
    repeated injury. A Prince whose character is thus marked by every act which
    may define a Tyrant, is unfit to be the ruler of a free people.

    Nor have We been wanting in attentions to our Brittish brethren. We have
    warned them from time to time of attempts by their legislature to extend an
    unwarrantable jurisdiction over us. We have reminded them of the
    circumstances of our emigration and settlement here. We have appealed to
    their native justice and magnanimity, and we have conjured them by the ties
    of our common kindred to disavow these usurpations, which, would inevitably
    interrupt our connections and correspondence. They too have been deaf to
    the voice of justice and of consanguinity. We must, therefore, acquiesce in
    the necessity, which denounces our Separation, and hold them, as we hold
    the rest of mankind, Enemies in War, in Peace Friends.

    We, therefore, the Representatives of the united States of America, in
    General Congress, Assembled, appealing to the Supreme Judge of the world
    for the rectitude of our intentions, do, in the Name, and by Authority of
    the good People of these Colonies, solemnly publish and declare, That these
    United Colonies are, and of Right ought to be Free and Independent States;
    that they are Absolved from all Allegiance to the British Crown, and that
    all political connection between them and the State of Great Britain, is
    and ought to be totally dissolved; and that as Free and Independent States,
    they have full Power to levy War, conclude Peace, contract Alliances,
    establish Commerce, and to do all other Acts and Things which Independent
    States may of right do. And for the support of this Declaration, with a
    firm reliance on the protection of divine Providence, we mutually pledge to
    each other our Lives, our Fortunes and our sacred Honor.
    """



# Copyright (C) 2017-2018 StARLinG Lab
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program (at the base of this repository). If not,
# see <http://www.gnu.org/licenses/>

"""
rnlp.parse
----------
"""

from __future__ import print_function
from __future__ import division

try:
   input = raw_input
except NameError:
   pass

# Standard Python Library
import string

# Non-standard Python Library
import nltk
from tqdm import tqdm
from .textprocessing import getSentences
from .textprocessing import getBlocks

def _writeBlock(block,blockID):
    '''writes the block to a file with the id'''
    with open("blockIDs.txt","a") as fp:
        fp.write("blockID: "+str(blockID)+"\n")
        sentences = ""
        for sentence in block:
            sentences += sentence+","
        fp.write("block sentences: "+sentences[:-1]+"\n")
        fp.write("\n")

def _writeSentenceInBlock(sentence,blockID,sentenceID):
    '''writes the sentence in a block to a file with the id'''
    with open("sentenceIDs.txt","a") as fp:
        fp.write("sentenceID: "+str(blockID)+"_"+str(sentenceID)+"\n")
        fp.write("sentence string: "+sentence+"\n")
        fp.write("\n")

def _writeWordFromSentenceInBlock(word,blockID,sentenceID,wordID):
    '''writes the word from a sentence in a block to a file with the id'''
    with open("wordIDs.txt","a") as fp:
        fp.write("wordID: "+str(blockID)+"_"+str(sentenceID)+"_"+str(wordID)+"\n")
        fp.write("wordString: "+word+"\n")
        fp.write("\n")

def _writeFact(predicateString):
    '''writes the fact to facts file'''
    with open("facts.txt","a") as f:
        f.write(predicateString+"\n")

def _writeBk(target="sentenceContainsTarget(+SID,+WID).", treeDepth="3",
            nodeSize="3", numOfClauses="8"):
    """
    Writes a background file to disk.

    :param target: Target predicate with modes.
    :type target: str.
    :param treeDepth: Depth of the tree.
    :type treeDepth: str.
    :param nodeSize: Maximum size of each node in the tree.
    :type nodeSize: str.
    :param numOfClauses: Number of clauses in total.
    :type numOfClauses: str.
    """

    with open('bk.txt', 'w') as bk:

        bk.write("useStdLogicVariables: true\n")

        bk.write("setParam: treeDepth=" + str(treeDepth) + '.\n')
        bk.write("setParam: nodeSize=" + str(nodeSize) + '.\n')
        bk.write("setParam: numOfClauses=" + str(numOfClauses) + '.\n')

        bk.write("mode: nextSentenceInBlock(+BID,+SID,-SID).\n")
        bk.write("mode: nextSentenceInBlock(+BID,-SID,+SID).\n")
        bk.write("mode: earlySentenceInBlock(+BID,-SID).\n")
        bk.write("mode: midWaySentenceInBlock(+BID,-SID).\n")
        bk.write("mode: lateSentenceInBlock(+BID,-SID).\n")
        bk.write("mode: sentenceInBlock(-SID,+BID).\n")
        bk.write("mode: wordString(+WID,#WSTR).\n")
        bk.write("mode: partOfSpeechTag(+WID,#WPOS).\n")
        bk.write("mode: nextWordInSentence(+SID,+WID,-WID).\n")
        bk.write("mode: earlyWordInSentence(+SID,-WID).\n")
        bk.write("mode: midWayWordInSentence(+SID,-WID).\n")
        bk.write("mode: lateWordInSentence(+SID,-WID).\n")
        bk.write("mode: wordInSentence(-WID,+SID).\n")

        bk.write("mode: " + target + "\n")

    return

def makeIdentifiers(blocks, target="sentenceContainsTarget(+SID,+WID).",
                    treeDepth="3", nodeSize="3", numOfClauses="8"):
    """
    Make unique identifiers for components of the block and write to files.

    :param blocks: Blocks of sentences (likely the output of
                   ``textprocessing.getBlocks``).
    :type blocks: list
    :param target: Target to write to the background file (another option might
                   be ``blockContainsTarget(+BID,+SID).``).
    :type target: str.
    :param treeDepth: Depth of the tree.
    :type treeDepth: str.
    :param nodeSize: Maximum size of each node in the tree.
    :type nodeSize: str.
    :param numOfClauses: Number of clauses in total.
    :type numOfClauses: str.

    .. note:: This is a function that writes *facts*, presently there is no
      way to distinguish between these and positive/negatives examples.

    Example:

    .. code-block:: python

                    from rnlp.textprocessing import getSentences
                    from rnlp.textprocessing import getBlocks
                    from rnlp.parse import makeIdentifiers

                    example = "Hello there. How are you? I am fine."

                    sentences = getSentences(example)
                    # ['Hello there', 'How are you', 'I am fine']

                    blocks = getBlocks(sentences, 2)
                    # with 1: [['Hello there'], ['How are you'], ['I am fine']]
                    # with 2: [['Hello there', 'How are you'], ['I am fine']]
                    # with 3: [['Hello there', 'How are you', 'I am fine']]

                    makeIdentifiers(blocks)
                    # 100%|██████████████████████| 2/2 [00:00<00:00, 18.49it/s]
    """
    blockID, sentenceID, wordID = 1, 0, 0
    #checkConsistency()
    print("Creating background file...")

    _writeBk(target=target, treeDepth=treeDepth,
             nodeSize=nodeSize, numOfClauses=numOfClauses)

    print("Creating identifiers from the blocks...")
    nBlocks = len(blocks)
    for block in tqdm(blocks):

        _writeBlock(block, blockID)

        sentenceID = 1
        nSentences = len(block)
        beginning = nSentences/float(3)
        ending = (2*nSentences)/float(3)

        for sentence in block:

            if sentenceID < nSentences:
                #=====================predicate: nextSentenceInBlock(blockID,sentenceID,sentenceID)======================
                predicateString = "nextSentenceInBlock("+str(blockID)+","+str(blockID)+"_"+str(sentenceID)+","+str(blockID)+"_"+str(sentenceID+1)+")."
                _writeFact(predicateString)
            #=====================predicate: earlySentenceInBlock(blockID,sentenceID)===========================
            if sentenceID < beginning:
                predicateString = "earlySentenceInBlock("+str(blockID)+","+str(blockID)+"_"+str(sentenceID)+")."
                _writeFact(predicateString)
            #=====================predicate: midWaySentenceInBlock(blockID,sentenceID)==========================
            if sentenceID >= beginning and sentenceID < ending:
                predicateString = "earlySentenceInBlock("+str(blockID)+","+str(blockID)+"_"+str(sentenceID)+")."
                _writeFact(predicateString)
            #=====================predicate: lateSentenceInBlock(blockID,sentenceID)============================
            if sentenceID > ending:
                predicateString = "lateSentenceInBlock("+str(blockID)+","+str(blockID)+"_"+str(sentenceID)+")."
                _writeFact(predicateString)
            #print("writing sentence "+str(sentenceID)+"/"+str(nSentences)+" in block id "+str(blockID)+" to sentenceIDs.txt..")
            #====================predicate: sentenceInBlock(sentenceID,blockID)=====================================
            predicateString = "sentenceInBlock("+str(blockID)+"_"+str(sentenceID)+","+str(blockID)+")."
            _writeFact(predicateString)
            _writeSentenceInBlock(sentence,blockID,sentenceID)
            sentenceID += 1
            wordID = 1
            tokens = nltk.word_tokenize(sentence)
            nWords = len(tokens)
            beginning = nWords/float(3)
            ending = (2*nWords)/float(3)
            for word in tokens:
                #============predicate: wordString(wordID,#str)==================
                '''
                if word == "you":
                    pos = open("pos.txt","a")
                    word = str(blockID)+"_"+str(sentenceID)+"_"+str(wordID)
                    sentence = str(blockID)+"_"+str(sentenceID)
                    pos.write("sentenceContainsTarget("+sentence+","+word+").\n")
                    pos.close()
                else:
                    neg = open("neg.txt","a")
                    word = str(blockID)+"_"+str(sentenceID)+"_"+str(wordID)
                    sentence = str(blockID)+"_"+str(sentenceID)
                    neg.write("sentenceContainsTarget("+sentence+","+word+").\n")
                    neg.close()
                '''
                predicateString = "wordString("+str(blockID)+"_"+str(sentenceID)+"_"+str(wordID)+","+"\'"+str(word)+"\')."
                _writeFact(predicateString)
                #============predicate: partOfSpeechTag(wordID,#POS)=============
                POS = nltk.pos_tag([word])[0][1]
                predicateString = "partOfSpeech("+str(blockID)+"_"+str(sentenceID)+"_"+str(wordID)+","+"\""+str(POS)+"\")."
                _writeFact(predicateString)
                if wordID < nWords:
                    #====================predicate: nextWordInSentence(sentenceID,wordID,wordID)==========================
                    predicateString = "nextWordInSentence("+str(blockID)+"_"+str(sentenceID)+","+str(blockID)+"_"+str(sentenceID)+"_"+str(wordID)+","+str(blockID)+"_"+str(sentenceID)+"_"+str(wordID+1)+")."
                    _writeFact(predicateString)
                #=====================predicate: earlyWordInSentence(sentenceID,wordID)===========================
                if wordID < beginning:
                    predicateString = "earlyWordInSentence("+str(blockID)+"_"+str(sentenceID)+","+str(blockID)+"_"+str(sentenceID)+"_"+str(wordID)+")."
                    _writeFact(predicateString)
                #=====================predicate: midWayWordInSentences(sentenceID,wordID)==========================
                if wordID >= beginning and wordID < ending:
                    predicateString = "midWayWordInSentence("+str(blockID)+"_"+str(sentenceID)+","+str(blockID)+"_"+str(sentenceID)+"_"+str(wordID)+")."
                    _writeFact(predicateString)
                #=====================predicate: lateWordInSentence(sentenceID,wordID)============================
                if sentenceID > ending:
                    predicateString = "lateSentenceInBlock("+str(blockID)+","+str(blockID)+"_"+str(sentenceID)+")."
                    _writeFact(predicateString)
                #print("writing word "+str(wordID)+"/"+str(nWords)+" from sentence id "+str(sentenceID)+" in block id "+str(blockID)+" to wordIDs.txt..")
                #====================predicate: wordInSentence(wordID,sentenceID)=====================================
                predicateString = "wordInSentence("+str(blockID)+"_"+str(sentenceID)+"_"+str(wordID)+","+str(blockID)+"_"+str(sentenceID)+")."
                _writeFact(predicateString)
                _writeWordFromSentenceInBlock(word,blockID,sentenceID,wordID)
                wordID += 1
        blockID += 1



# Copyright (C) 2017-2018 StARLinG Lab
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program (at the base of this repository). If not,
# see <http://www.gnu.org/licenses/>

"""
rnlp
====

``pip install rnlp``

**Relational-NLP Preprocessing** (rnlp) is a Python package for converting text
corpus into a set of relational facts.

Designed for use with BoostSRL (https://github.com/starling-lab/BoostSRL).

Motivation
==========

There is no doubt that natural language processing (nlp) is a difficult task,
but recent work has kindled interest in modeling the context or relations
between words and their meanings in high-dimensional spaces.

The intent of this package is to help with building systems which explicitly
model words as a series of entities, relations, and attributes on those
relations.

If words are entities, then their position in a certain document is as relation
between the word and a document entity, their context is a set of relations
between the word entity and the other words it appears near, their attributes
may be the part of speech, etc.

Examples
========

.. code-block:: python

                import rnlp
                import rnlp.corpus

                # Get a document from the built-in corpus
                declaration = rnlp.corpus.declaration()

                # Convert the document into a set of relational facts
                predicates = rnlp.parse(declaration)
"""

__author__ = 'Kaushik Roy (@kkroy36)'
__copyright__ = 'Copyright (c) 2017-2018 StARLinG Lab'
__license__ = 'GPL-v3'

__version__ = '0.2.1'
__status__ = 'Beta'
__maintainer__ = 'Alexander L. Hayes (@batflyer)'
__email__ = 'alexander.hayes@utdallas.edu'

__credits__ = [
    'Kaushik Roy (@kkroy36)',
    'Alexander L. Hayes (@batflyer)',
    'Sriraam Natarajan (@boost-starai)',
    'Gautam Kunapuli (@gkunapuli)',
    'Dileep Viswanathan',
    'Rahul Pasunuri'
]

from . import parse

from .textprocessing import getBlocks
from .textprocessing import getSentences

def converter(input_string, block_size=2):
    """
    The cli tool as a built-in function.

    :param input_string: A string that should be converted to a set of facts.
    :type input_string: str.
    :param blocks_size: Optional block size of sentences (Default: 2).
    :type block_size: int.
    """

    sentences = textprocessing.getSentences(input_string)
    blocks = textprocessing.getBlocks(sentences, block_size)
    parse.makeIdentifiers(blocks)



# Copyright (C) 2017-2018 StARLinG Lab
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program (at the base of this repository). If not,
# see <http://www.gnu.org/licenses/>

"""
textprocessing
--------------

A set of functions for normalizing text, with options for stemming,
stopping, removing punctuation, etc.

Document Hierarchy
------------------

A corpora is a collection of documents.
A document is a collection of chapters.
A chapter is a collection of paragraphs.
A paragraph is a collection of sentences.
A sentence is a collection of words.
A word is a collection of letters...

The depth of reasoning probably depends on the domain you are working on.
"""

from nltk import word_tokenize
from nltk import sent_tokenize
from nltk.corpus import stopwords
from nltk.stem.porter import *

import string

_punctuation = string.punctuation
_stemmer = PorterStemmer()
_stopwords = stopwords.words('english')

def __removePunctuation(text_string):
    """
    Removes punctuation symbols from a string.

    :param text_string: A string.
    :type text_string: str.

    :returns: The input ``text_string`` with punctuation symbols removed.
    :rtype: str.

    >>> from rnlp.textprocessing import __removePunctuation
    >>> example = 'Hello, World!'
    >>> __removePunctuation(example)
    'Hello World'
    """
    return text_string.strip(_punctuation)

def __removeStopwords(text_list):
    """
    Removes stopwords contained in a list of words.

    :param text_string: A list of strings.
    :type text_string: list.

    :returns: The input ``text_list`` with stopwords removed.
    :rtype: list
    """

    output_list = []

    for word in text_list:
        if word.lower() not in _stopwords:
            output_list.append(word)

    return output_list

def getBlocks(sentences, n):
    """
    Get blocks of n sentences together.

    :param sentences: List of strings where  each string is a sentence.
    :type sentences: list
    :param n: Maximum blocksize for sentences, i.e. a block will be composed of
              ``n`` sentences.
    :type n: int.

    :returns: Blocks of n sentences.
    :rtype: list-of-lists

    .. code-block:: python

                    import rnlp

                    example = "Hello there. How are you? I am fine."

                    sentences = rnlp.getSentences(example)
                    # ['Hello there', 'How are you', 'I am fine']

                    blocks = rnlp.getBlocks(sentences, 2)
                    # with 1: [['Hello there'], ['How are you'], ['I am fine']]
                    # with 2: [['Hello there', 'How are you'], ['I am fine']]
                    # with 3: [['Hello there', 'How are you', 'I am fine']]
    """
    blocks = []
    for i in range(0, len(sentences), n):
        blocks.append(sentences[i:(i+n)])
    return blocks

def getSentences(text_string):
    """
    Tokenizes the corpus into sentences, removing punctuation as it does so.

    :param text_string: A string.
    :type text_string: str.

    :returns: A list of string sentences with punctuation removed.
    :rtype: list

    .. code-block:: python

                    import rnlp

                    example = "Hello there. How are you? I am fine."
                    sentences = rnlp.getSentences(example)
                    # ['Hello there', 'How are you', 'I am fine']
    """
    return [__removePunctuation(s) for s in sent_tokenize(text_string)]



"""
Setup file for rnlp
"""

from setuptools import setup, find_packages
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))

# Get the long description from the README.md
with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

# rnlp needs a few additional nltk packages to function properly.
import nltk
nltk.download('punkt')
nltk.download('stopwords')
#nltk.download('wordnet')
nltk.download('averaged_perceptron_tagger')

# Import the package and assign metadata as appropriate
import rnlp

setup(
    name='rnlp',
    version=rnlp.__version__,
    license=rnlp.__license__,

    description='Converts text corpora into a set of relational facts.',
    long_description=long_description,

    url='https://github.com/starling-lab/rnlp',

    author='Alexander L. Hayes (@batflyer)',
    author_email='alexander@batflyer.net',

    classifiers=[
        # Development Information
        'Development Status :: 4 - Beta',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',

        # Intended primarily for research use
        'Intended Audience :: Science/Research',

        # Tested Operating Systems
        'Operating System :: POSIX :: Linux',
        'Operating System :: Microsoft :: Windows :: Windows 10',

        # Supported Python Versions.
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6'
    ],

    keywords='nlp',

    project_urls={
        'Source': 'https://github.com/starling-lab/rnlp',
        'Tracker': 'https://github.com/starling-lab/rnlp/issues'
    },

    packages=find_packages(exclude=['tests'])
)
